<?php

namespace ShineSoftware\Constriv\Gateway\Request;

use Magento\Payment\Gateway\ConfigInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use Magento\Store\Model\ScopeInterface;
use ShineSoftware\Constriv\Model\Source\CurrenciesFactory;

class VoidRequest implements BuilderInterface {

    private $config;
    protected $_sourceCurrenciesFactory;

    public function __construct(
    ConfigInterface $config,
    CurrenciesFactory $sourceCurrenciesFactory
    ) {
        $this->config = $config;
        $this->_sourceCurrenciesFactory = $sourceCurrenciesFactory;
    }

    public function build(array $buildSubject) {
        if (!isset($buildSubject['payment']) || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new \InvalidArgumentException('Payment data object should be provided');
        }
        $paymentDO = $buildSubject['payment'];
        $order = $paymentDO->getOrder();
        $currencies = $this->_sourceCurrenciesFactory->create();
        $payment = $paymentDO->getPayment();
        if (!$payment instanceof OrderPaymentInterface) {
            throw new \LogicException('Order payment should be provided.');
        }
        $addInfo = $payment->getAdditionalInformation(Transaction::RAW_DETAILS);
        if (empty($addInfo)) {
            throw new \Exception('No Authorization transaction information have been found!');
        }
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/password', ScopeInterface::SCOPE_STORE),
            'action' => 9,
            'currencycode' => $currencies->findCurrencyIsoCode($payment->getOrder()->getOrderCurrencyCode()),
            'amt' => number_format($order->getGrandTotal(), 2, '.', ''),
            'paymentid' => $addInfo['paymentid'],
            'transid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
        ];
        return $data;
    }

}

;
